;****************************************************************************
; 	Clignoteur1led.asm
;	Mini-carte avec 5 leds et 1 BP
;	TMR0 dclenche une interruption qui dcrmente un compteur toutes les 0,066 secondes
;	Le sous-programme Dlai permet de temporiser simplement entre 0,066s et 255*0,066s
;	On fait clignoter la led0
;	3 dcembre 2007

;*************** DEFINITION & CONFIGURATION PROFONDE DU C ******************
	#include <p12F683.inc>
	__config (_INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _IESO_OFF & _FCMEN_OFF & _BOD_OFF )

;****************** DEFINITIONS POUR LE COMPILATEUR *************************
; "Raccoucis d'criture"
	#define	BANK0		bcf	STATUS,RP0	; Selection Bank0
	#define	BANK1		bsf	STATUS,RP0	; Selection Bank1

;Noms des Entres/Sorties
	#define	Leds		GPIO
	#define	Led0		GPIO,0
	#define	Led1		GPIO,1
	#define	Led2		GPIO,2
	#define	BP			GPIO,3
	#define	Led4		GPIO,4
	#define	Led5		GPIO,5

;******************	VARIABLES ******************

	cblock	0x70	; Variables "multiBanque" seulement 16 sont disponibles
		WTEMP				; Sauvegarde de W
		STATUSTEMP			; et de STATUS quand il y a une interruption
		Compteur			; Comptage du temps (0,1s)
	endc

;**************************** ADRESSE DE DEMARRAGE **************************
	ORG     0x000             ; Adresse du vecteur de RAZ
	goto    Init              ; branchement au dbut du programme

;**************************** TRAITEMENT DES INTERRUPTIONS ******************
	ORG		0x004			; Adresse du vecteur d'interruption
	movwf	WTEMP			; Sauvegarde de W et de STATUS
	movf	STATUS,w
	movwf	STATUSTEMP

;-------------------- Traitement de l'interruption TMR0 ---------------------
IntTMR0
	decf	Compteur,f		; Compteur auxiliaire pour avoir des multiples de 0,1s

FinIntTMR0
	bcf		INTCON,T0IF		; on autorise de nouveau l'interruption

	movf	STATUSTEMP,w	; Remise en tat de W et de STATUS
	movwf	STATUS
	swapf	WTEMP,f
	swapf	WTEMP,w
	retfie

	
;****** CONFIGURATION & INITIALISATION DES E/S, Horloge, Timers... *******

Init
; Initialisation des ports
	BANK0
	clrf	GPIO		; Initialise le port
	movlw	b'00000111'
	movwf	CMCON0		; Les E/S du comparateur non relies et digitales
	BANK1
	clrf	ANSEL		; pas d'entre analogique!
	movlw	b'00001000'
	movwf	TRISIO		; Toutes les broches en sortie (sauf RA3 bien sr)

; Choix d'une horloge interne de 31kHz lente
	movlw	b'00000001'
	movwf	OSCCON

; Configuration de Timer0
	movlw	b'00000000'	; Utilise l'horloge interne (31kHz)
	movwf	OPTION_REG	; Prescaler = 2
						; Une interruption est gnre toutes les 2*256/(31000/4) = 0,066s

; Initialisation des variables

;Autorisation des interruptions	
	movlw	b'10100000'	; Autorise les interruptions dues 
	movwf	INTCON		; TMR0

;**************************** PROGRAMME PRINCIPAL ******************
;	
Main
	BANKSEL	Leds
Main1
	bsf		Led0		; On allume la led 0
	movlw	.1			; On dtermine un dlai de 1 x 0,066s
	call	Delai		; On lance l'excution de la temporisation

	bcf		Led0		; On teint la led 0
	movlw	D'7'		; puis on refait la mme chose - D'7' signifie que le nombre est
	call	Delai		; exprim en dcimal
	goto	Main1

;************************** SOUS-PROGRAMMES *************************

Delai	
; Temporisation de W x 0,066 seconde
; W est inchang, Compteur est modifi!
	movwf	Compteur		; On charge la valeur dans Compteur
							; qui est automatiquement dcrment toutes les 0,066s par le SP d'interruption
Delai1
	movf	Compteur,f		;
	btfss	STATUS,Z		; On attend le passage  0 de Compteur
	goto	Delai1	
	return

;***************************** FIN ***************************
	end
